
<cfcomponent
	output="false"
	hint="I provide access to the task data.">
	
	
	<cffunction 
		name="init" 
		access="public" 
		returntype="any" 
		output="false" 
		hint="I initialize the component.">
	
		<!--- Define arguments. --->
		<cfargument 
			name="dataPath" 
			type="string" 
			required="true" 
			hint="I am the data path to which the task data is persisted to file."
			/>
	
		<!--- Store variables. --->
		<cfset variables.dataPath = arguments.dataPath />
		
		<!--- 
			Create the internal query that is going to store all of 
			the task information. This will basically act as our 
			internal database of tasks.
		--->
		<cfset variables.tasks = queryNew( 
			"id, task, dueDate, taskList, user",
			"cf_sql_varchar, cf_sql_varchar, cf_sql_timestamp, cf_sql_varchar, cf_sql_varchar"
			) />
			
		<!--- Load any persisted data into the query. --->
		<cfset this.loadData() />
	
		<!--- Return this object reference. --->	
		<cfreturn this />
	</cffunction>
	
	
	<cffunction 
		name="deleteTask" 
		access="public" 
		returntype="string" 
		output="false" 
		hint="I delete the task with the given ID.">
	
		<!--- Define arguments. --->	
		<cfargument 
			name="id" 
			type="string" 
			required="true" 
			hint="I am the unique ID of the given task."
			/>
		
		<!--- Define the local scope. --->
		<cfset var local = {} />
		
		<!--- 
			Remove the given task from the local query. This will select
			all the non-matching tasks and overwrite the local query.
		--->
		<cfquery name="variables.tasks" dbtype="query">
			SELECT
				*
			FROM
				variables.tasks
			WHERE
				id != <cfqueryparam value="#arguments.id#" cfsqltype="cf_sql_varchar" />
		</cfquery>
		
		<!--- Persist the updated the task data. --->
		<cfset this.saveData() />
		
		<!--- Return the original task id. --->
		<cfreturn arguments.id />
	</cffunction>
	
	
	<cffunction 
		name="getTask" 
		access="public" 
		returntype="query" 
		output="false" 
		hint="I return the task with the given ID.">
	
		<!--- Define arguments. --->	
		<cfargument 
			name="id" 
			type="string" 
			required="true" 
			hint="I am the unique ID of the given task."
			/>
		
		<!--- Define the local scope. --->
		<cfset var local = {} />
		
		<!--- Query for the given task. --->
		<cfquery name="local.task" dbtype="query">
			SELECT
				*
			FROM
				variables.tasks
			WHERE
				id = <cfqueryparam value="#arguments.id#" cfsqltype="cf_sql_varchar" />
		</cfquery>
		
		<!--- Return the given task. --->
		<cfreturn local.task />
	</cffunction>
	
	
	<cffunction 
		name="getTasksByList" 
		access="public" 
		returntype="query" 
		output="false" 
		hint="I return the tasks associated with the given list and user.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="taskList" 
			type="string" 
			required="true" 
			hint="I the task list name."
			/>
			
		<cfargument 
			name="user" 
			type="string" 
			required="true" 
			hint="I the user who owns the task list."
			/>
		
		<!--- Define the local scope. --->
		<cfset var local = {} />
		
		<!--- Query for the tasks for this list owned by this user. --->
		<cfquery name="local.tasks" dbtype="query">
			SELECT
				id,
				task,
				dueDate,
				taskList,
				[user]
			FROM
				variables.tasks
			WHERE
				taskList = <cfqueryparam value="#arguments.taskList#" cfsqltype="cf_sql_varchar" />
			AND
				[user] = <cfqueryparam value="#arguments.user#" cfsqltype="cf_sql_varchar" />
			ORDER BY
				dueDate ASC
		</cfquery>
		
		<!--- Return the tasks. --->
		<cfreturn local.tasks />
	</cffunction>
	
	
	<cffunction 
		name="getTaskListsByUser" 
		access="public" 
		returntype="query" 
		output="false" 
		hint="I return the task lists associated with the given user.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="user" 
			type="string" 
			required="true" 
			hint="I the user."
			/>
		
		<!--- Define the local scope. --->
		<cfset var local = {} />
		
		<!--- Query for the unique task lists for this user. --->
		<cfquery name="local.taskLists" dbtype="query">
			SELECT
				taskList,
				COUNT( * ) AS taskCount
			FROM
				variables.tasks
			WHERE
				[user] = <cfqueryparam value="#arguments.user#" cfsqltype="cf_sql_varchar" />
			GROUP BY
				taskList
			ORDER BY
				taskList ASC		
		</cfquery>
		
		<!--- Return the task lists. --->
		<cfreturn local.taskLists />
	</cffunction>
	
	
	<cffunction 
		name="loadData" 
		access="public" 
		returntype="void" 
		output="false" 
		hint="I load any persisted data from the local file system into the task query.">
	
		<!--- Define the local scope. --->
		<cfset var local = {} />
		
		<!--- Read in the WDDX file if it exists. --->
		<cfif fileExists( variables.dataPath )>
		
			<!--- Read the file content. --->
			<cfset local.wddxData = fileRead( variables.dataPath ) />
			
			<!--- 
				Convert the WDDX data into a raw query object. Due to the 
				way in which WDDX serializes data, we don't want to use this
				query directly. Rather, we wnat to use it to build another 
				query that will be more appropriately data-typed.
			--->
			<cfwddx 
				output="local.rawQuery"
				action="wddx2cfml"
				input="#local.wddxData#"
				/>
				
			<!--- 
				Create a intermediate query that we are going to create by 
				column based on teh de-serialized data and the meta data of 
				the original query.
			--->
			<cfset local.queryByColumn = queryNew( "" ) />
			
			<!--- 
				Get the meta data of the original query - this will be use 
				used to enforce the type integrity of our intermediary query.
			--->
			<cfloop
				index="local.columnMetaData"
				array="#getMetaData( variables.tasks )#">
				
				<!--- 
					Add the data column to the intermediary query. This will use 
					the data type of the original query and the data from the 
					de-serialized WDDX query.
				--->
				<cfset queryAddColumn(
					local.queryByColumn,
					local.columnMetaData.name,
					local.columnMetaData.typeName,
					local.rawQuery[ local.columnMetaData.name ]
					) />
				
			</cfloop>
				
			<!--- Now that we have fixed the query, store it locally. --->
			<cfset variables.tasks = local.queryByColumn />
		
		</cfif>
		
		<!--- Return out. --->
		<cfreturn />
	</cffunction>
	
	
	<cffunction 
		name="saveData" 
		access="public" 
		returntype="void" 
		output="false" 
		hint="I persist the task data to the local file system.">
	
		<!--- Define the local scope. --->
		<cfset var local = {} />
	
		<!--- Convert the task query to WDDX data. --->
		<cfwddx
			output="local.wddxData"
			action="cfml2wddx"
			input="#variables.tasks#"
			/>
		
		<!--- Write the WDDX data to the file system. --->
		<cfset fileWrite( variables.dataPath, local.wddxData ) />
		
		<!--- Return out. --->
		<cfreturn />
	</cffunction>
	
	
	<cffunction 
		name="saveTask" 
		access="public" 
		returntype="string" 
		output="false" 
		hint="I save the given task and return a ID of the saved task.">
	
		<!--- Define arguments. --->	
		<cfargument 
			name="id" 
			type="string" 
			required="false" 
			default="" 
			hint="I am the unique ID of the task. If this is a new task, then leave blank."
			/>
			
		<cfargument 
			name="task" 
			type="string" 
			required="true" 
			hint="I am the task description."
			/>
		
		<cfargument 
			name="dueDate" 
			type="date" 
			required="true" 
			hint="I am the due date of the task." 
			/>
			
		<cfargument 
			name="taskList" 
			type="string"
			required="true" 
			hint="I am the name of task list to which this task belongs."
			/>
		
		<cfargument 
			name="user" 
			type="string" 
			required="true" 
			hint="I am the owner (user) that created this task."
			/>
		
		<!--- Define the local scope. --->
		<cfset var local = {} />
		
		<!--- 
			When saving this task, we might be creating OR updating a given 
			task. As such, we need to loop over the query to see if we can 
			find a matching row with the given ID.	
		--->
		<cfloop query="variables.tasks">
		
			<!--- Check to see if the ID matches this row. --->
			<cfif (variables.tasks.id eq arguments.id)>
			
				<!--- We found an existing, matching task. Update the values. --->
				<cfset variables.tasks[ "task" ][ variables.tasks.currentRow ] = javaCast( "string", arguments.task ) />
				<cfset variables.tasks[ "dueDate" ][ variables.tasks.currentRow ] = parseDateTime( arguments.dueDate ) />
				<cfset variables.tasks[ "taskList" ][ variables.tasks.currentRow ] = javaCast( "string", arguments.taskList ) />
				<cfset variables.tasks[ "user" ][ variables.tasks.currentRow ] = javaCast( "string", arguments.user ) />
				
				<!--- Persist the updated the task data. --->
				<cfset this.saveData() />
				
				<!--- We found a matching row, so return the given ID. --->
				<cfreturn arguments.id />
				
			</cfif>
		
		</cfloop>
		
		
		<!--- 
			If we made it this far, then we were not able to find a row 
			that matched the given ID. As such, we need to insert one.
		--->
		<cfset queryAddRow( variables.tasks ) />
		
		<!--- Create a new ID for the task. --->
		<cfset arguments.id = createUUID() />
		
		<!--- Save the new task information. --->
		<cfset variables.tasks[ "id" ][ variables.tasks.recordCount ] = javaCast( "string", arguments.id ) />
		<cfset variables.tasks[ "task" ][ variables.tasks.recordCount ] = javaCast( "string", arguments.task ) />
		<cfset variables.tasks[ "dueDate" ][ variables.tasks.recordCount ] = parseDateTime( arguments.dueDate ) />
		<cfset variables.tasks[ "taskList" ][ variables.tasks.recordCount ] = javaCast( "string", arguments.taskList ) />
		<cfset variables.tasks[ "user" ][ variables.tasks.recordCount ] = javaCast( "string", arguments.user ) />
		
		<!--- Persist the updated the task data. --->
		<cfset this.saveData() />
				
		<!--- Return the new ID. --->
		<cfreturn arguments.id />		
	</cffunction>
	
</cfcomponent>